/*
 *  ====================================================================
 *    Licensed to the Apache Software Foundation (ASF) under one or more
 *    contributor license agreements.  See the NOTICE file distributed with
 *    this work for additional information regarding copyright ownership.
 *    The ASF licenses this file to You under the Apache License, Version 2.0
 *    (the "License"); you may not use this file except in compliance with
 *    the License.  You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 * ====================================================================
 */

//package org.apache.poi.xslf.usermodel.tutorial;

import org.apache.poi.xslf.usermodel.SlideLayout;
import org.apache.poi.xslf.usermodel.XMLSlideShow;
import org.apache.poi.xslf.usermodel.XSLFSlide;
import org.apache.poi.xslf.usermodel.XSLFSlideLayout;
import org.apache.poi.xslf.usermodel.XSLFSlideMaster;
import org.apache.poi.xslf.usermodel.XSLFTextShape;
//import org.apache.poi.xslf.usermodel.Placeholder;
import org.apache.poi.xslf.usermodel.XSLFPictureData;
import org.apache.poi.xslf.usermodel.XSLFPictureShape;
import org.apache.commons.io.IOUtils;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.awt.Rectangle;
import java.awt.Toolkit;


/**
 * Create slides from pre-defined slide layouts
 *
 * @author Ritesh Sood
 */
public class SlideEditor {
    
	XMLSlideShow ppt;
	String tmplFname;
	String outFname;

	double res;
	double width;
	double height;

	double pgr_x;
	double pgr_y;
	XSLFSlideMaster defaultMaster;

	public SlideEditor(String tmpl_fname, String out_fname)  throws Exception{
		
		tmplFname = tmpl_fname;
		outFname = out_fname;
		
		res = Toolkit.getDefaultToolkit().getScreenResolution();
		width = Toolkit.getDefaultToolkit().getScreenSize().width;
		height = Toolkit.getDefaultToolkit().getScreenSize().height;
                           
		//Display screen resolution, width, and height                                   
		System.out.println("Screen resolution: " + res + " dpi");
		System.out.println("Screen width: " + width + " dots");
		System.out.println("Screen height: " + height + " dots");
		System.out.println("----------------------------------");
		
		ppt = new XMLSlideShow(new FileInputStream(tmplFname));
		java.awt.Dimension pgSize = ppt.getPageSize();
		System.out.println("Page size: " + pgSize);
		// onscreen the page size is shown to be always 10" x 7.5"
		// the page ratio (pgr) is the ratio of onscreen size vs 
		// the page size we got above  
		pgr_x = 10.0*res/pgSize.getWidth();
		pgr_y = 7.5*res/pgSize.getHeight();
		System.out.println("----------------------------------");
		 
		// first see what slide layouts are available by default
        System.out.println("Slide layout tpyes and names:");
        for(XSLFSlideMaster master : ppt.getSlideMasters()){
            for(XSLFSlideLayout layout : master.getSlideLayouts()){
                System.out.println("Name: " + layout.getName());
                System.out.println("Type: " + layout.getType());
            }
			System.out.println("-----------");
        }
        
		defaultMaster = ppt.getSlideMasters().get(0);  
		
	}

	public void addTitleSlide()  throws Exception{
        
        // title slide
        //XSLFSlideLayout titleLayout = defaultMaster.getLayout(SlideLayout.TITLE_ONLY);
        XSLFSlideLayout titleLayout = defaultMaster.getLayout("Title Only");
        XSLFSlide slide1 = ppt.createSlide(titleLayout);
        XSLFTextShape title1 = slide1.getPlaceholder(0);
		title1.setText("First Title");

	}

	public void addContentSlide()  throws Exception{
        // title and content
        //XSLFSlideLayout titleBodyLayout = defaultMaster.getLayout(SlideLayout.TITLE_AND_CONTENT);
        XSLFSlideLayout titleBodyLayout = defaultMaster.getLayout("Title and Content");
        XSLFSlide slide2 = ppt.createSlide(titleBodyLayout);

        XSLFTextShape title2 = slide2.getPlaceholder(1);
        title2.setText("Second Title");

        XSLFTextShape body2 = slide2.getPlaceholder(0);
        body2.clearText(); // unset any existing text
        body2.addNewTextParagraph().addNewTextRun().setText("First paragraph");
        body2.addNewTextParagraph().addNewTextRun().setText("Second paragraph");
        body2.addNewTextParagraph().addNewTextRun().setText("Third paragraph");
	
	}
	
	public void addFigSlide(String fig_fname, String fig_format, String title)  throws Exception{
		
		// title and picture (Thor)
        //XSLFSlideLayout titleBodyLayout3 = defaultMaster.getLayout(SlideLayout.TITLE_ONLY);
        //XSLFSlideLayout titleBodyLayout3 = defaultMaster.getLayout("Title Only");

		// title and picture (Atlas)
        //XSLFSlideLayout titleBodyLayout3 = defaultMaster.getLayout(SlideLayout.CUST);
        XSLFSlideLayout titleBodyLayout3 = defaultMaster.getLayout("Picture Slide");
	
        XSLFSlide slide3 = ppt.createSlide(titleBodyLayout3);

		System.out.println("Slide placeholders:");
		String tt = "test"; 
		for(XSLFTextShape ph : slide3.getPlaceholders()){
			tt = ph.getText();
			System.out.println("Text: " + tt);

        }
		System.out.println("-----------");
  
        XSLFTextShape title3 = slide3.getPlaceholder(0);
        title3.setText(title);

        //XSLFTextShape body3 = slide3.getPlaceholder(1);
        //body3.clearText(); // unset any existing text
        
		byte[] pictureData = IOUtils.toByteArray(new FileInputStream(fig_fname));
		XSLFPictureData pd; 
		if (fig_format == "svg")
			pd = ppt.addPicture(pictureData, XSLFPictureData.PictureType.SVG);
		else
			pd = ppt.addPicture(pictureData, XSLFPictureData.PictureType.PNG);

		XSLFPictureShape pic = slide3.createPicture(pd);

        int o_x = (int) Math.round(1.0*res/pgr_x);
        int o_y = (int) Math.round(1.0*res/pgr_y);
        int d_x = (int) Math.round(8*res/pgr_x);
        int d_y = (int) Math.round(6*res/pgr_y);
        System.out.println("Picture size set to: ");
		System.out.println("o_x : " + o_x);
		System.out.println("o_y : " + o_y);
		System.out.println("d_x : " + d_x);
		System.out.println("d_y : " + d_y);
		 
        Rectangle anchor = new Rectangle(o_x, o_y, d_x, d_y);
        pic.setAnchor(anchor);
	
	}
        
	public void close()  throws Exception{
	
		FileOutputStream out = new FileOutputStream(outFname);
        ppt.write(out);
        out.close();
	}

}

